#!/usr/bin/env python
#
#   sprites.py
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Defines game sprites"""

import pygame
from pygame.locals import *

from vector import Vector

__author__ = 'Ross Light'
__date__ = 'May 22, 2006'
__all__ = ['getImage',
           'ImageManager',
           'Sprite',
           'Animation',]
__docformat__ = 'reStructuredText'

def getImage(image, tryIM=True):
    """
    Retrieves an image.
    
    Provides a standard way to either pass an image or a string to a function.
    """
    if isinstance(image, pygame.Surface):
        return image
    else:
        if tryIM:
            try:
                return ImageManager.loadImage(image)
            except ValueError:
                pass
        return pygame.image.load(image)

class ImageManager(object):
    """
    Singleton class to manage all of the game's images.
    
    As with all of the singleton classes, you must call the setup method before
    using the class.
    """
    
    @classmethod
    def setup(self):
        self.imagePaths = {}
        self.imageCache = {}
    
    @classmethod
    def prepare(self, tag, defaultPath=None):
        """
        Declares a tag for later use.
        
        Although you don't have to use prepare before loading an image, it is
        recommended you do so.  `loadImage` will become very confused unless the
        tags you are using are the exact path of the image (which I **really**
        don't recommend).
        """
        if defaultPath is None:
            defaultPath = tag
        self.imagePaths[tag] = defaultPath
    
    @classmethod
    def cache(self, tag):
        """
        Caches the tag and returns the image.
        
        This will be done automatically by `loadImage` if the ``cache`` flag is
        ``True``, but you may not want your users to have to have a delay when
        you use a new resource.
        """
        if tag in self.imageCache:
            image = self.imageCache[tag]
        else:
            path = self.imagePaths.get(tag, tag)
            image = pygame.image.load(path).convert_alpha()
            self.imageCache[tag] = image
        return image
    
    @classmethod
    def uncache(self, tag):
        """Removes the image denoted by ``tag`` from the cache."""
        try:
            del self.imageCache[tag]
        except KeyError:
            pass
    
    @classmethod
    def loadImage(self, tag, cache=True):
        """
        Loads an image from disk, using a cached representation, if possible.
        
        The ``cache`` flag specifies whether the image should be saved to the
        cache, not whether the cache will be used.
        
        This method raises a ``ValueError`` if the image cannot be loaded.
        
        See also:
        
        - `prepare`
        - `cache`
        """
        if tag in self.imageCache:
            return self.imageCache[tag]
        else:
            if cache:
                return self.cache(tag)
            else:
                path = self.imagePaths.get(tag, tag)
                return pygame.image.load(path).convert_alpha()

class Sprite(pygame.sprite.Sprite, object):
    """Abstract superclass for sprites."""
    hpadding = vpadding = 0
    angleTolerance = 0.5
    clamp = True
    
    def __init__(self, image=None):
        """
        Initializes a sprite.
        
        If the ``image`` parameter is omitted, the class variable ``image`` will
        be used.  If the parameter it finally receives is a string, it will try
        to load the image from the `ImageManager`.  If it is not in the
        `ImageManager`, it assumes that it is a path or pygame.Surface.
        """
        pygame.sprite.Sprite.__init__(self)
        if image is None:
            image = self.image
        self.setImage(image)
        self.rect = self.image.get_rect()
        self.area = pygame.display.get_surface().get_rect()
        self.angle = 0.0
    
    def setImage(self, image, tryIM=True):
        """Changes the current image and the revert image variables."""
        self.image = self._image = getImage(image, tryIM).convert_alpha()
    
    def collideBox(self):
        """
        Returns the box used for checking with the `touches` method.
        
        By default, this uses the ``hpadding`` and ``vpadding`` to construct an
        inset box.  Override to have a different collide box.
        """
        # Multiply by two to get all-around coverage
        return self.rect.inflate(self.hpadding * -2, self.vpadding * -2)
    
    def touches(self, other):
        """
        Returns whether the other sprite is actually touching the receiver.
        """
        return self.collideBox().colliderect(other.collideBox())
    
    def updateWithVector(self, v, clamp=None):
        """Moves the sprite with the given vector."""
        self.rect.x += v.x
        self.rect.y += v.y
        if clamp is None:
            clamp = self.clamp
        if clamp:
            self.rect = self.rect.clamp(self.area)
    
    def rotateImage(self):
        """Rotates the sprite's image to the proper angle."""
        if abs(self.angle) < self.angleTolerance:
            self.image = self._image
        else:
            self.image = pygame.transform.rotate(self._image, self.angle)
        self.rect.size = self.image.get_size()

class Animation(Sprite):
    """Superclass for ambient animations."""
    loop = False
    
    def __init__(self, frames=None, loop=None):
        """
        Initializes an animation.
        
        - ``frames`` is a list of pygame.Surfaces or strings.  If not specified,
          it uses the ``frames`` class variable.
        - ``loop`` is a flag specifying whether the animation should
          continuously play or whether it should kill itself after one play.  If
          not specified, it uses the ``loop`` class variable.
        """
        if frames is None:
            frames = self.frames
        self.frames = list(frames)
        if loop is not None:
            self.loop = loop
        self.frameNum = 0
        super(Animation, self).__init__(frames[0])
    
    def update(self):
        self.advance()
    
    def advance(self):
        """Advances to the next frame and dies if it reaches the end."""
        self.frameNum += 1
        if self.frameNum >= len(self.frames):
            if self.loop:
                self.frameNum = 0
            else:
                self.kill()
        else:
            self.setImage(self.frames[self.frameNum])



#!/usr/bin/env python

import sys

import pygame
from pygame.locals import *
import pymage

class HelloState(pymage.states.State):
    quitOnEscape = True
    
    def __init__(self):
        # Call superclass implementation, using a white background
        super(HelloState, self).__init__((255, 255, 255))
        # Perform initialization here...
    
    def handle(self, event):
        super(HelloState, self).handle(event)
        # Handle event here...
    
    def update(self, game):
        # Update game state here...
        pass
    
    def firstDisplay(self, screen):
        super(HelloState, self).firstDisplay(screen)
        # Draw state for first time here and call pygame.display.flip()
    
    def display(self, screen):
        screen.fill(self.bgColor)
        image = pymage.sprites.ImageManager.loadImage('Hello')
        rect = pygame.Rect(0, 0, image.get_width(), image.get_height())
        rect.center = screen.get_rect().center
        screen.blit(image, rect)
        pygame.display.flip()

class BasicGame(pymage.states.Game):
    screenSize = (640, 480)
    # Uncomment to run fullscreen
    #flags = FULLSCREEN
    
    def preloop(self):
        pymage.config.setup()                       # Read gamesite.xml file
        pygame.mouse.set_visible(False)             # Hide cursor
        pygame.display.set_caption("Hello World")
        self.changeToState(HelloState())

if __name__ == '__main__':
    BasicGame(sys.argv[0]).run()



#!/usr/bin/env python
#
#   __init__.py
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Various utilities to simplify pygame development."""

__author__ = 'Ross Light'
__date__ = 'July 20, 2006'
__all__ = ['config',
           'joystick',
           'sound',
           'sprites',
           'states',
           'timer',
           'ui',
           'vector',]
__docformat__ = 'reStructuredText'

import config
import joystick
import sound
import sprites
import states
import timer
import ui
import vector



#!/usr/bin/env python
#
#   states.py
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Manages game states (e.g. paused, levels...)"""

import os
import sys

import pygame
from pygame.locals import *

import sprites
import sound
import ui

__author__ = 'Ross Light'
__date__ = 'May 22, 2006'
__all__ = ['State', 'Paused', 'Game']
__docformat__ = 'reStructuredText'

class State(object):
    """Game state that handles events and displays on a surface."""
    bgColor = (0, 0, 0)
    quitOnEscape = False
    
    def __init__(self, bgColor=None):
        if bgColor is not None:
             self.bgColor = bgColor
    
    def handle(self, event):
        """
        Hook method for handling events.
        
        This implementation *should* always be called.
        """
        if event.type == QUIT:
            sys.exit()
        elif self.quitOnEscape and \
             event.type == KEYDOWN and \
             event.key == K_ESCAPE:
            sys.exit()
    
    def update(self, game):
        """
        Hook method for updating the state.
        
        ``game`` is an instance of `Game`.
        """
        pass
    
    def firstDisplay(self, screen):
        """
        Hook method for displaying the `State` for the first time.
        
        By default, this merely fills the screen with the background color.
        """
        screen.fill(self.bgColor)
        pygame.display.flip()       # Swap buffers
    
    def display(self, screen):
        """Hook method for drawing the `State`."""
        pass

class Paused(State):
    """
    Abstract superclass for simple pause screens that can show text and/or an
    image.
    """
    finished = False
    image = None
    useIM = True
    text = ""
    fontSize = 48
    textColor = (0, 0, 0)
    showCursor = False
    excludedKeys = []
    
    def __init__(self,
                 bgColor=None,
                 text=None,
                 image=None,
                 fontSize=None,
                 useIM=None):
        super(Paused, self).__init__(bgColor)
        # Set up text
        if text is not None:
            self.text = text
        if fontSize is not None:
            self.fontSize = fontSize
        # Set up image
        if image is not None:
            self.image = image
        if useIM is not None:
            self.useIM = useIM
        if self.useIM and self.image is not None:
            self.image = sprites.ImageManager.loadImage(self.image)
    
    def handle(self, event):
        """Stops the pause screen after a click or key press."""
        super(Paused, self).handle(event)
        if event.type in (MOUSEBUTTONUP, JOYBUTTONUP) or \
           event.type == KEYUP and event.key not in self.excludedKeys:
            self.finished = True
    
    def update(self, game):
        if self.finished:
            pygame.mouse.set_visible(False)
            game.changeToState(self.nextState())
    
    def firstDisplay(self, screen):
        screen.fill(self.bgColor)   # Clear screen
        font = pygame.font.Font(None, self.fontSize)
        lines = self.text.strip().splitlines()
        # Calculate positioning
        height = len(lines) * font.get_linesize()
        center, top = screen.get_rect().center
        top -= height // 2
        # Show image
        if self.image:
            if isinstance(self.image, basestring):
                image = pygame.image.load(self.image).convert_alpha()
            else:
                image = self.image
            imageRect = image.get_rect()
            top += imageRect.height // 2
            imageRect.midbottom = center, top - 20
            screen.blit(image, imageRect)   # show the image
        # Render lines
        antialias = True
        for line in lines:
            text = font.render(line.strip(), antialias, self.textColor)
            textRect = text.get_rect()
            textRect.midtop = center, top
            screen.blit(text, textRect)
            top += font.get_linesize()
        # Show cursor (if necessary)
        pygame.mouse.set_visible(self.showCursor)
        pygame.display.flip()   # swap buffers
    
    def nextState(self):
        """
        Hook method for advancing to the next state.
        
        Raises a ``NotImplementedError`` if not overridden.
        """
        raise NotImplementedError("Override nextState")

class Menu(State):
    """Abstract superclass for menu screens."""
    def __init__(self, *args, **kw):
        super(Menu, self).__init__(*args, **kw)
        initRect = pygame.display.get_surface().get_rect()
        self.newState = None
        self.container = ui.Container(bgColor=self.bgColor,
                                      rect=initRect)
        self.cursor = None
        self.body()
    
    def body(self):
        """
        Adds the menu's contents.
        
        Override in subclasses.  The root widget is the ``container`` attribute.
        You may set the ``cursor`` attribute to a `ui.CursorWidget`.
        """
        pass
    
    def handle(self, event):
        super(Menu, self).handle(event)
        self.container.processEvent(event)
        if self.cursor is not None:
            self.cursor.handle(event)
    
    def update(self, game):
        """
        Updates the UI widgets.
        
        If you change the ``newState`` attribute to a non-``None`` value, it
        will become the new state.
        """
        self.container.update()
        if self.cursor is not None:
            self.cursor.update()
        if self.newState is not None:
            game.changeToState(self.newState)
            self.newState = None
    
    def firstDisplay(self, screen):
        self.container.refresh()
        self.container.rect.size = screen.get_size()
        super(Menu, self).firstDisplay(screen)
    
    def display(self, screen):
        super(Menu, self).display(screen)
        updates = self.container.display(screen)
        if self.cursor is not None:
            updates += self.cursor.display(screen)
        pygame.display.update(updates)

class Game(object):
    """Game object that manages the state machine."""
    screenSize = (800, 600)
    ticks = 60
    flags = 0
    
    def __init__(self, rootDir, **kw):
        # Change to root directory (for relative paths)
        os.chdir(os.path.abspath(os.path.dirname(rootDir)))
        # Start with no state
        self.state = self.nextState = None
        # Set up instance variables
        self.__dict__.update(kw)
    
    def changeToState(self, state):
        """
        Changes to the given `State` instance on the next event loop iteration.
        """
        self.nextState = state
    
    def run(self):
        """
        Run main event loop.
        
        You *shouldn't* have to override this...
        """
        pygame.init()
        self.screen = pygame.display.set_mode(self.screenSize, self.flags)
        self.clock = pygame.time.Clock()
        self.preloop()
        while True:
            self.iterate()
        self.postloop()
    
    def preloop(self):
        """Hook method to do something before the event loop starts."""
        pass
    
    def iterate(self):
        """
        Called to perform one iteration of the event loop.
        
        You *shouldn't* have to override the core functionality, but you may
        wish to override this method to do something before or after.
        """
        # Constant FPS, Potter!
        self.clock.tick(self.ticks)
        # Change to the new state (if there is one)
        if self.state != self.nextState:
            self.state = self.nextState
            self.state.firstDisplay(self.screen)
        # Send events to the state
        for event in pygame.event.get():
            if event.type == sound.MusicManager.endEvent:
                sound.MusicManager.nextSong()
            else:
                self.state.handle(event)
        # Update state
        self.state.update(self)
        # Display state
        self.state.display(self.screen)
    
    def postloop(self):
        """Hook method to do something after the event loop exits."""
        pass



#!/usr/bin/env python
#
#   vectortest.py
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

__author__ = 'Ross Light'
__date__ = 'July 26, 2006'
__all__ = ['VectorTestCase', 'suite']

import unittest

from pymage.vector import *

class VectorTestCase(unittest.TestCase):
    def setUp(self):
        self.v1 = Vector(4.2, 8.7)          # Our test 2D vector
        self.v2 = Vector(1.125, 1.25, 3.14) # Our test 3D vector
    
    def testVectorFind(self):
        ang, mag = 42.7, 5.7
        v = Vector.findVector(ang, mag)
        # Float math is almost always inaccurate.  So let's almost equal our
        # expected result!
        self.assertAlmostEqual(v.angle, ang, msg="Angle is incorrect")
        self.assertAlmostEqual(v.magnitude, mag, msg="Magnitude is incorrect")
    
    def testImmutable(self):
        oldX = self.v1.x
        self.assertRaises(AttributeError,
                          setattr,
                          self.v1,
                          'x',
                          42)
        self.assertEqual(oldX, self.v1.x,
                         "Variable changed")
    
    def testHashable(self):
        d = {self.v1: False,
             Vector(*self.v1): True}
        self.assertEqual(len(d.keys()), 1, "Invalid hashes")
        # This will test whether the other value overwrote the other.  With
        # Python, using the same key twice sets the value to last defined
        # value (which is True, in this case).
        self.assert_(d[self.v1], "Didn't get the proper value")
    
    def testAngle(self):
        self.assertAlmostEqual(Vector.angleBetween(i, j),
                               90,
                               msg="Perpendicular is not 90 degrees")
    
    def testIter(self):
        self.assertEqual(list(self.v1), [self.v1.x, self.v1.y],
                         "2D vector does not give proper iterator")
        self.assertEqual(list(self.v2), [self.v2.x, self.v2.y, self.v2.z],
                         "3D vector does not give proper iterator")
    
    def testTruth(self):
        self.assert_(not bool(Vector()), "Zero vector is True")
        self.assert_(bool(self.v1), "Nonzero vector is False")
    
    def testEquality(self):
        self.assertEqual(self.v1, Vector(*self.v1),
                         "2D vector is not equal to copy")
        self.assertEqual(self.v2, Vector(*self.v2),
                         "3D vector is not equal to copy")
    
    def testInequality(self):
        self.assertNotEqual(self.v1, self.v2,
                            "2D vector is equal to 3D vector")
    
    def testAddSub(self):
        self.assertEqual(self.v1 + self.v2 - self.v2, self.v1,
                         "Adding and subtracting is non-functional")
    
    def testMulDiv(self):
        scalar = 3.14
        self.assertEqual(self.v1 * scalar / scalar, self.v1,
                         "Multiplying and dividing is non-functional")
    
    def testUnitVector(self):
        self.assertAlmostEqual(self.v1.unitVector().magnitude, 1,
                               msg="Unit vectors do not have length of 1")

suite = unittest.makeSuite(VectorTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#!/usr/bin/env pythonw
#
#   sound.pyw
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""pymage sound subsystem"""

import pygame
from pygame.locals import *

__author__ = 'Ross Light'
__date__ = 'July 19, 2006'
__all__ = ['MusicManager',
           'SoundManager',]
__docformat__ = 'reStructuredText'

class MusicManager(object):
    """
    Singleton class to manage the game's music.
    
    For sound effects, use `SoundManager`.
    
    Before you can use this class, you need to call the `setup` method.
    Playlists then need to be prepared with the `prepare` method, and started
    with the `startPlaylist` method.
    
    Typical order of calls:
    
    1. setup
    2. prepare
    3. startPlaylist
    4. play
    """
    endEvent = 25
    
    @classmethod
    def setup(self,
              shouldPlay=True,
              volume=0.5,
              loop=True,):
        self.tagPlaylists = {}
        self.playlist = []
        self.loop = loop
        self.shouldPlay = shouldPlay
        self.setVolume(volume)
        self.index = 0
        self.playing = False
        pygame.mixer.music.set_endevent(self.endEvent)
    
    @classmethod
    def prepare(self, tag, playlist):
        """Prepares a playlist for playback later."""
        self.tagPlaylists[tag] = list(playlist)
    
    @classmethod
    def startPlaylist(self, tag):
        """
        Starts a playlist denoted by ``tag``.
        
        You may pass an iterable object to `startPlaylist`, and the object will
        be used as a playlist.
        """
        try:
            self.playlist = self.tagPlaylists[tag]
        except KeyError:
            try:
                self.playlist = list(tag)
            except TypeError:
                raise TypeError("Tag must be prepared or an iterable object")
        if self.playlist:
            self.index = 0
            self.loadSong()
    
    @classmethod
    def play(self):
        """Starts the current song."""
        if self.shouldPlay and not self.playing:
            if pygame.mixer.music.get_busy():
                pygame.mixer.music.unpause()
            else:
                pygame.mixer.music.play()
            self.playing = True
    
    @classmethod
    def pause(self):
        """
        Pauses the current song.
        
        You can resume using `play`.
        """
        pygame.mixer.music.pause()
        self.playing = False
    
    @classmethod
    def stop(self):
        """Stops the current song and goes to the beginning."""
        pygame.mixer.music.stop()
        self.playing = False
    
    @classmethod
    def loadSong(self, songFile=None):
        """
        Manually loads a song.
        
        If no argument is given, the current song in the playlist is loaded.
        This method will not load anything unless the manager is configured to
        play.
        """
        if songFile is None:
            if self.playlist:
                songFile = self.playlist[self.index]
            else:
                raise ValueError("No song in the playlist")
        if self.shouldPlay:
            pygame.mixer.music.load(songFile)
    
    @classmethod
    def previousSong(self):
        """Returns to the previous song in the playlist."""
        if self.playlist:
            self.index -= 1
            if self.index < 0:
                self.index = len(self.playlist) - 1
            self.stop()
            self.loadSong()
        
    @classmethod
    def nextSong(self):
        """Advances to the next song in the playlist."""
        if self.playlist:
            playNextSong = self.playing
            self.index += 1
            if self.index >= len(self.playlist):
                self.index = 0
                if not self.loop:
                    playNextSong = False
            self.stop()
            self.loadSong()
            if playNextSong:
                self.play()
    
    @classmethod
    def getVolume(self):
        """Gets the current volume."""
        return self._volume
    
    @classmethod
    def setVolume(self, volume):
        """Changes the current volume."""
        self._volume = volume
        pygame.mixer.music.set_volume(self._volume)

class SoundManager(object):
    """
    Singleton class to manage sound effects.
    
    For music, use `MusicManager`.
    
    As with `MusicManager`, this is a singleton class, so use the `setup` method
    before using it.
    """    
    @classmethod
    def setup(self,
              shouldPlay=True,
              volume=0.5):
        self.shouldPlay = shouldPlay
        self.volume = volume
        self.soundPaths = {}
        self.soundCache = {}
    
    @classmethod
    def prepare(self, tag, defaultPath=None):
        """
        Declares a tag for later use.
        
        Although you don't have to use `prepare` before loading a sound, it is
        recommended you do so.  `getSound` will become very confused unless the
        tags you are using are the exact path of the sound (which I **really**
        don't recommend).
        """
        if defaultPath is None:
            defaultPath = tag
        self.soundPaths[tag] = defaultPath
    
    @classmethod
    def cache(self, tag):
        """
        Caches the tag and returns the sound.
        
        This will be done automatically by `getSound` if the ``cache`` flag is
        ``True``, but you may not want your users to have to have a delay when
        you use a new resource.
        """
        if tag in self.soundCache:
            sound = self.soundCache[tag]
        else:
            path = self.soundPaths.get(tag, tag)
            sound = pygame.mixer.Sound(path)
            self.soundCache[tag] = sound
        return sound
    
    @classmethod
    def uncache(self, tag):
        """Removes the sound denoted by ``tag`` from the cache."""
        try:
            del self.soundCache[tag]
        except KeyError:
            pass
    
    @classmethod
    def getSound(self, tag, cache=True):
        """
        Retrieves a sound, using a cache if possible.
        
        The ``cache`` flag specifies whether the sound will be cached, not
        whether it uses the cache.
        """
        if tag in self.soundCache:
            return self.soundCache[tag]
        else:
            if cache:
                return self.cache(tag)
            else:
                path = self.soundPaths.get(tag, tag)
                return pygame.mixer.Sound(path)
    
    @classmethod
    def play(self, tag, volume=None, cache=True):
        """
        Plays a sound and returns the sound object.
        
        This method will use a cached representation if possible.  Also, you can
        specify a non-default volume, if desired.  The cache flag specifies
        whether the sound will be cached, not whether it uses the cache.
        """
        if volume is None:
            volume = self.volume
        if self.shouldPlay:
            snd = self.getSound(tag, cache)
            snd.set_volume(volume)
            snd.play()
            return snd
        else:
            return None



#!/usr/bin/env python
#
#   setup.py
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""setuptools script for pymage"""

from setuptools import setup
import sys

__author__ = 'Ross Light'
__date__ = 'August 16, 2006'
__all__ = []

if sys.version < '2.4.0':
    print >> sys.stderr, "PyMage requires Python 2.4"
    sys.exit(1)

setup(name="pymage",
      version='0.1.2',
      description="Pygame helper package",
      author="Ross Light",
      author_email='rlight2@gmail.com',
      keywords="pymage pygame",
      url='http://code.google.com/p/pymage/',
      test_suite='tests',
      packages=['pymage'],)



#!/usr/bin/env python
#
#   vector.py
#   pymage
#
#   Created by Ross Light on 3/3/06.
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""Manipulate mathematical vectors."""

from __future__ import division
import math

__author__ = 'Ross Light'
__date__ = 'March 3, 2006'
__all__ = ['Vector', 'i', 'j']
__docformat__ = 'reStructuredText'

scalarClasses = (int, float, long)

class Vector(object):
    """
    An three-dimensional vector.
    
    However, some methods assume it is two-dimensional (e.g. `angleBetween`).
    """
    __slots__ = ['x', 'y', 'z', '_magnitude', '_angle']
    
    def __init__(self, x=0, y=0, z=0):
        """
        ``Vector(x, y, z)`` -> `Vector`
        
        You don't have to specify all three coordinates, only those necessary.
        """
        super(Vector, self).__setattr__('x', float(x))
        super(Vector, self).__setattr__('y', float(y))
        super(Vector, self).__setattr__('z', float(z))
    
    @classmethod
    def findVector(cls, angle, magnitude):
        """
        Finds the two-dimensional vector with the given angle and magnitude.
        
        The angle is in degrees counterclockwise from the positive x-axis.
        Round-off may experience round-off error.
        """
        # Put angle in range of [0, 360)
        while angle < 0:
            angle += 360
        while angle >= 360:
            angle -= 360
        # Find angle in radians
        radAngle = math.radians(angle)
        # Catch special cases
        if angle == 90 or angle == 270:
            cos = 0
        else:
            cos = math.cos(radAngle)
        if angle == 0 or angle == 180:
            sin = 0
        else:
            sin = math.sin(radAngle)
        # Create vector
        return cls(cos * magnitude, sin * magnitude)
    
    @classmethod
    def twoPointVector(klass, p1, p2):
        """
        Finds the vector with the given points.
        
        The points can be one-, two-, or three-dimensional.
        """
        return klass(*p2) - klass(*p1)
    
    # String conversion
    
    def __repr__(self):
        cls = type(self)
        name = '%s.%s' % (cls.__module__, cls.__name__)
        if not self:
            return '%s()' % (name)
        elif self.z == 0:
            return '%s(%g, %g)' % (name, self.x, self.y)
        else:
            return '%s(%g, %g, %g)' % (name, self.x, self.y, self.z)
    
    def __str__(self):
        if self.z == 0:
            return '<%g, %g>' % (self.x, self.y)
        else:
            return '<%g, %g, %g>' % (self.x, self.y, self.z)
    
    # Operators
    
    def __neg__(self):
        return type(self)(-self.x, -self.y, -self.z)
    
    def __pos__(self):
        return type(self)(+self.x, +self.y, +self.z)
    
    def __add__(u, v):
        if isinstance(v, Vector):
            return type(u)(u.x + v.x, u.y + v.y, u.z + v.z)
        else:
            return NotImplemented
    
    def __sub__(u, v):
        if isinstance(v, Vector):
            return type(u)(u.x - v.x, u.y - v.y, u.z - v.z)
        else:
            return NotImplemented
        
    def __mul__(v, other):
        """
        ``v * x <==> v.__mul__(x)``
        
        x can be a scalar (scalar multiplication) or a `Vector` (dot product).
        """
        if isinstance(other, scalarClasses):    # Scalar
            s = float(other)
            return type(v)(v.x * s, v.y * s, v.z * s)
        elif isinstance(other, Vector):         # Dot Product
            u, v = v, other # For algorithm simplicity
            return u.x * v.x + u.y * v.y + u.z * v.z
        else:
            return NotImplemented
    
    def __rmul__(v, other):
        return v.__mul__(other)
    
    def __div__(v, other):
        """
        ``v / x <==> v.__div__(x)``
        
        ``x`` can be a scalar (scalar division) or a `Vector` (dot dividend).
        """
        # Do true division by default, because that's what one would expect,
        # since our instance variables are floats.
        return v.__truediv__(other)
        
    def __truediv__(v, other):
        """
        ``v / x <==> v.__truediv__(x)``
        
        ``x`` can be a scalar (scalar division) or a `Vector` (dot dividend).
        """
        if isinstance(other, scalarClasses):    # Scalar
            s = float(other)
            return v.__class__(v.x / s, v.y / s, v.z / s)
        elif isinstance(other, Vector):         # Dot Product
            u, v = v, other # For algorithm simplicity
            return u.x / v.x + u.y / v.y + u.z / v.z
        else:
            return NotImplemented
        
    def __floordiv__(v, other):
        """
        ``v // x <==> v.__floordiv__(x)``
        
        ``x`` can be a scalar (scalar division) or a `Vector` (dot dividend).
        """
        if isinstance(other, scalarClasses):    # Scalar
            s = float(other)
            return v.__class__(v.x // s, v.y // s, v.z // s)
        elif isinstance(other, Vector):         # Dot Product
            u, v = v, other # For algorithm simplicity
            return u.x // v.x + u.y // v.y + u.z // v.z
        else:
            return NotImplemented
    
    def proj(v, u):
        """Mathematically: projv u"""
        return (u * v / v.magnitude ** 2) * v
    
    def angleBetween(u, v):
        """Obtains the angle between the two vectors in degrees."""
        return math.degrees(math.acos((u * v) / (u.magnitude * v.magnitude)))
    
    def unitVector(self):
        """Creates a vector in the same direction, but of magnitude 1."""
        return self / self.magnitude
    
    # List conversion
    
    def __iter__(self):
        """
        ``iter(v) <==> v.__iter__()``
        
        In sequence, this returns the components of the vector, and leaving z
        out if it is 0.  This is usually what you want, but you can force 2D or
        3D by using `iter2D` and `iter3D`, respectively.
        """
        yield self.x
        yield self.y
        if self.z != 0:
            yield self.z
    
    def iter2D(self):
        """Returns an iterator that accesses only the x and y components."""
        yield self.x
        yield self.y
    
    def iter3D(self):
        """Returns an iterator that accesses the x, y, and z components."""
        yield self.x
        yield self.y
        yield self.z
    
    # Comparison
    
    def __eq__(u, v):
        if isinstance(v, Vector):
            return u.x == v.x and u.y == v.y and u.z == v.z
        else:
            return NotImplemented
    
    def __ne__(u, v):
        if isinstance(v, Vector):
            return u.x != v.x or u.y != v.y or u.z != v.z
        else:
            return NotImplemented
    
    def __nonzero__(self):
        return bool(self.x or self.y or self.z)
    
    def __hash__(self):
        return int(self.x) ^ int(self.y) ^ int(self.z)
    
    # Component access
    
    def __setattr__(self, attr, value):
        raise AttributeError("Vector is an immutable object")
    
    def __delattr__(self, attr):
        raise AttributeError("Vector is an immutable object")
    
    def _getMagnitude(self):
        try:
            return self._magnitude
        except AttributeError:
            mag = math.sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)
            super(Vector, self).__setattr__('_magnitude', mag)
            return mag
    
    def _getAngle(self):
        try:
            return self._angle
        except AttributeError:
            # Find angle in degrees, ignore z-axis
            if self.x == 0:
                if self.y > 0:
                    deg = 90.0
                elif self.y < 0:
                    deg = -90.0
                else:
                    deg = 0.0
            else:
                deg = math.degrees(math.atan(self.y / self.x))
                while deg < 0:
                    deg += 360
                while deg > 360:
                    deg -= 360
            super(Vector, self).__setattr__('_angle', deg)
            return deg
    
    magnitude = property(_getMagnitude, doc="The length of the vector")
    angle = property(_getAngle,
                     doc="The angle in degrees from the positive x-axis.")

# Special vectors
i = Vector(1, 0)
j = Vector(0, 1)



#!/usr/bin/env python
#
#   config.py
#   pymage
#
#   Copyright (C) 2006 Ross Light
#
#   This library is free software; you can redistribute it and/or
#   modify it under the terms of the GNU Lesser General Public
#   License as published by the Free Software Foundation; either
#   version 2.1 of the License, or (at your option) any later version.
#   
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   Lesser General Public License for more details.
#   
#   You should have received a copy of the GNU Lesser General Public
#   License along with this library; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
#   USA
#

"""
Provide configuration for your game.

The typical use of this module is to use the `setup` function and give set up
parameters.  From there, the module sets up the correct parts of the pymage
library.

This module is not strictly necessary, but pymage's reasoning is as follows:

Resources for the game should be loaded on an location-independent basis.
This provides abstraction and the ability for arbitrary file structure.
However, these files eventually need to be located, so the developer should
create a file (separate from the main code) that locates everything.  From
there, there may be a legitimate reason for the user to want to customize the
file locations (perhaps he/she wants the game resources to be on a separate hard
drive).  Therefore, a separate file is used to override the behavior of any
given installation of the game.  But since the user never touches the
developer's default game setup, any corrupt preferences can be corrected by
deleting the user configuration files.

Here is a sample gamesite.xml file::

    <?xml version="1.0"?>
    
    <!-- name is not necessary, but reserved for future use. -->
    <game-site name="Sample Game">
        <!-- Prepare an image in the ImageManager.
             The id attribute is used as the tag.
             The path element is required to find the file.
             The section and option elements specify how the path can be
             overriden in configuration files. -->
        <image id="SampleImage">
            <section>sprites</section>
            <option>sample</option>
            <path>img/sample.png</path>
        </image>
        
        <!-- Prepare a sound in the SoundManager.
             The id attribute is used as the tag.
             The path element is required to find the file.
             The section and option elements specify how the path can be
             overriden in configuration files. -->
        <sound id="SampleSound">
            <section>sounds</section>
            <option>sample</option>
            <path>sfx/sample.wav</path>
        </sound>
        
        <!-- Prepare a playlist in the MusicManager.
             The id attribute is used as the tag.
             The path element(s) are required to find the music file(s).
             The section and option elements specify how the playlist can be
             overriden in configuration files. -->
        <playlist id="SamplePlaylist">
            <section>playlists</section>
            <option>sample</option>
            <path>music/song1.wav</path>
            <path>music/song2.wav</path>
        </playlist>
        
        <!-- Specify additional configuration files. -->
        <config-file>userconfig.ini</config-file>
    </game-site>
"""

from ConfigParser import ConfigParser
import os
from textwrap import dedent
from xml.dom import minidom

from sound import SoundManager, MusicManager
from sprites import ImageManager

__author__ = 'Ross Light'
__date__ = 'August 10, 2006'
__all__ = ['load',
           'save',
           'getOption',
           'setOption',
           'setup']
__docformat__ = 'reStructuredText'

class CaseConfigParser(ConfigParser):
    """A ``ConfigParser`` that is case-sensitive."""
    def optionxform(self, optstr):
        return optstr

def load(*args, **kw):
    """
    Load a series of configuration files.
    
    This function returns a dictionary of values.  Any optional keyword values
    are used as variables.  The special keyword ``convert`` specifies whether
    the function should interpret the values as what they seem to be
    (e.g. ``float``, ``int``).  The default is ``True``.
    """
    # Retrieve convert keyword
    convertValues = kw.get('convert', True)
    try:
        del kw['convert']
    except KeyError:
        pass
    # Parse the files
    parser = CaseConfigParser(kw)
    for f in args:
        close = False
        if isinstance(f, basestring):
            # Open strings as paths
            path = os.path.normpath(os.path.expanduser(f))
            if os.path.exists(path):
                f = open(path)
            else:
                continue
            close = True
        parser.readfp(f)
        if close:
            f.close()
    # Assemble dictionary
    d = {}
    for section in parser.sections():
        sectDict = {}
        for option in parser.options(section):
            value = parser.get(section, option)
            if convertValues:   # Interpret values
                value = getValue(value)
            sectDict[option] = value
        d[section] = sectDict
    return d

def save(config, configFile):
    """Saves a configuration dictionary to a file."""
    close = False
    parser = CaseConfigParser()
    for section, values in config.iteritems():
        for option, value in values.iteritems():
            parser.set(section, option, value)
    if isinstance(configFile, basestring):
        path = os.path.normpath(os.path.expanduser(configFile))
        configFile = open(path, 'w')
        close = True
    parser.write(configFile)
    if close:
        configFile.close()

def getValue(s):
    """Retrieves a value from a ``ConfigParser`` string."""
    boolLiterals = {'false': False,
                    'no': False,
                    'off': False,
                    'true': True,
                    'yes': True,
                    'on': True,}
    if s.isdigit():                 # Integer
        return int(s)
    elif s.lower() in boolLiterals: # Boolean
        return boolLiterals[s.lower()]
    elif isFloat(s):                # Float
        return float(s)
    else:                           # String
        return str(s)

def isFloat(s):
    """
    Returns whether the string is a ``float``.
    
    The format for a float is::
    
        int[.[fraction]]
    """
    if s.isdigit():         # int
        return True
    if s.count('.') == 1:
        s = s.replace('.', '')
        if s.isdigit():     # int[.[fraction]]
            return True
        else:
            return False
    else:
        return False

def getOption(config, section, option, default=None):
    """Retrieves an option from the configuration dictionary."""
    try:
        section = config[section]
    except KeyError:
        return default
    else:
        return section.get(option, default)

def setOption(config, section, option, value):
    """Changes an option in the configuration dictionary."""
    section = config.setdefault(section, {})
    section[option] = value

def setup(site='gamesite.xml', *configFiles):
    """
    Sets up a game from the specified parameters and returns the configuration
    dictionary.
    
    The first argument is the game site file.  The game site file contains most
    of the setup information.  It specifies the resource IDs and where to find
    the configuration files.  It should only be modified by developers as the
    game is created.
    
    The additional arguments are program default configuration files.  These
    are parsed before any inside the game site file (therefore giving the site
    configuration files higher precedence).
    """
    images, sounds, playlists, siteConfigs = parseGameSite(site)
    configFiles = list(configFiles) + list(siteConfigs)
    config = load(*configFiles)
    ImageManager.setup()
    SoundManager.setup(bool(getOption(config, 'sound', 'play', True)),
                       float(getOption(config, 'sound', 'volume', 1.0)),)
    MusicManager.setup(bool(getOption(config, 'music', 'play', True)),
                       float(getOption(config, 'music', 'volume', 0.5)),
                       bool(getOption(config, 'music', 'loop', True)),)
    # Set up images
    for tag, section, option, path in images:
        if option is None:
            # Using default path
            ImageManager.prepare(tag, path)
        else:
            # Using specified section/option
            if section is None:
                section = 'sprites'
            ImageManager.prepare(tag, getOption(config, section, option, path))
    # Set up sounds
    for tag, section, option, path in sounds:
        if option is None:
            # Using default path
            SoundManager.prepare(tag, path)
        else:
            # Using specified section/option
            if section is None:
                section = 'sound'
            SoundManager.prepare(tag, getOption(config, section, option, path))
    # Set up playlists
    for tag, section, option, paths in playlists:
        if option is None:
            # Using default path
            MusicManager.prepare(tag, paths)
        else:
            # Using specified section/option
            if section is None:
                section = 'music'
            oldPaths = paths
            paths = getOption(config, section, option, paths)
            if paths is not oldPaths:
                # Found the option successfully
                paths = paths.split(',')
            del oldPaths
            MusicManager.prepare(tag, paths)
    # Return config dictionary
    return config

def parseGameSite(f):
    doc = minidom.parse(f)
    images = []
    sounds = []
    playlists = []
    configs = []
    for child in doc.documentElement.childNodes:
        if child.nodeType == minidom.Node.ELEMENT_NODE:
            if child.tagName == 'image':
                images.append([child.getAttribute('id'),
                               getText(childNamed(child, 'section')),
                               getText(childNamed(child, 'option')),
                               getText(childNamed(child, 'path')),])
            elif child.tagName == 'sound':
                sounds.append([child.getAttribute('id'),
                               getText(childNamed(child, 'section')),
                               getText(childNamed(child, 'option')),
                               getText(childNamed(child, 'path')),])
            elif child.tagName == 'playlist':
                tag = child.getAttribute('id')
                section = getText(childNamed(child, 'section'))
                option = getText(childNamed(child, 'option'))
                paths = [getText(sub) for sub in child.childNodes if
                         sub.nodeType == minidom.Node.ELEMENT_NODE and
                         sub.tagName == 'path']
                playlists.append([tag, section, option, paths])
            elif child.tagName == 'config-file':
                configs.append(getText(child))
    return images, sounds, playlists, configs

def getText(elem, post=True):
    if elem is None:
        return None
    text = ''
    for child in elem.childNodes:
        if child.nodeType == minidom.Node.TEXT_NODE:
            text += child.wholeText
    if post:
        text = dedent(text)
        if text.startswith('\n'):
            text = text[1:]
            if text.endswith('\n'):
                text = text[:-1]
    return text

def childNamed(elem, name):
    for child in elem.childNodes:
        if (child.nodeType == minidom.Node.ELEMENT_NODE and
            child.tagName == name):
            return child
    else:
        return None
